# Modelowanie przeprowadzone w kolejnosci proponowanej przez J. Farawaya:
# diagnostyka obs odstajacych, transformacje, selekcja cech
# (str 134 "Practical Regression and ANOVA using R" - ksiazka dostepna w sieci)
library(MASS); X=na.omit(Cars93[,c(2,7,12:15,17,19,21,22,24,25)])
row.names(X)=X[,1]; X=X[,-1]
# 1. Diagnostyka obserwacji odstajacych - wersja uproszczona
m1=lm(MPG.city~.,data=X)
x11(); par(mfrow=c(1,2))
res=rstudent(m1); hii=hatvalues(m1)
plot(hii,res,type="n"); abline(h=0,col=2)
text(hii,res,as.character(1:nrow(X)),cex=.75)
plot(m1$fit,X[,"MPG.city"],type="n"); abline(0,1,col=2)
text(m1$fit,X[,"MPG.city"],as.character(1:nrow(X)),cex=.75)
# Testuje teraz hipoteze H0 == "nie ma obserwacji odstajacych" na poziomie 
# alfa=0.1 metoda Bonferroniego, czyli wykonuje ciag testow  T[i] dla hipotez
# H[i] == "i-ta obs nie odstaje" na poziomie alfa/n, gdzie n - liczba obs.
# Wiemy, ze res[i] ~ t(n-p-1), gdzie p - liczba param modelu.
# Zatem do testowania H[i] mozemy wykorzystac (dwustronny) test t-studenta.
# Obliczamy wartosc krytyczna 
t0=qt(.1/(2*nrow(X)),nrow(X)-ncol(X)-1)  
#  =-3.371733.Nastepnie sprawdzamy, ze nie ma residuum przekraczajacego te wartosc.
# Zatem nie odrzucamy H0. Ogolnie: 
X=X[ -which( abs(res) > -t0 ) , ] 

# 2. Transformacje zmiennych
x11(); par(mfrow=c(4,6))
for(i in 1:11)plot(density(X[,i]),main=colnames(X)[i])
plot(1,1,type="n",axes=F,xlab="",ylab="") # dla symetrii miedzy X[,i] i log(X[,i]
for(i in 1:11)plot(density(log(X[,i])),main=paste("log-",colnames(X)[i]))
# widac, ze transformacja log(X) zbliza cechy 2:5 do  "normalnosci". Zatem

X[,2:5]=log(X[,2:5]) # MPG zmienie po "konsultacji" z logtrans i boxcox:
library(MASS); x11(); par(mfrow=c(1,2))
logtrans(MPG.city~.,data=X,alpha=seq(-15.5,0,len=100))
# logtrans uzasadnia przeksztalcenie log(X[,1]-12)
boxcox(MPG.city-12~.,data=X,lambda=seq(-2,2,len=100))
# boxcox potwierdza log(X[,1]-12)
X[,1]=log(X[,1]-12)
colnames(X)[1:5]=c("logMPG.city","logEngineSize",
                   "logHorsepower","logRPM","logRev.per.mile")

# 3. Selekcja cech
print(round(cor(X),2)) 
# Sa cechy b ze soba skorelowane, czyli prawie wspolliniowe. 
print("**************************")

m2=lm(logMPG.city~.,data=X) 
print(summary(m2))
print("**************************")
# Najistotniejsza jest logPojemn, ale tylko na "." - chyba z powodu wspolliniowosci.
# Do takiego samego modelu prowadzi krokowa optymalizacja bayesowskiego kryterium 
# informacyjnego (BIC): wykonuje stepAIC z paramentrem k= log(n).
m3=stepAIC(m2,scope=list(upper=.~.,lower=.~1),trace=T,k=log(nrow(X)))
print("**************************")
print(summary(m3))
break
# Nastepnie sprawdzam istotnosc pozostalych cech, usuwajac po jednej:
m4=update(m3,.~.-logHorsepower); print(dropterm(m4,test="F")); print(summary(m4))  
print("**************************")
m5=update(m3,.~.-Fuel.tank.capacity);  print(dropterm(m5,test="F")); print(summary(m5))
print("**************************")
m6=update(m5,.~.-logHorsepower); print(dropterm(m6,test="F")); print(summary(m6)) 
print("**************************")
# Wybieram najprostrzy model m6: logMPG.city ~ logEngineSize, 
# z cor =.9, bo traci tylko .02 do m2
print(round(cor(X[,"logMPG.city"],m6$fit),2)) 



